# instanzio un oggetto simulatore
set ns [new Simulator]

# definisco i colori per le classi
$ns color 1 Blue
$ns color 2 Red

# definisco il tipo di simulazione
# !! e' solo per la versione di ns2 installata in laboratorio EDA
$ns set cosim_type 0

# creo un file per NAM con le statistiche
set nf [open out.nam w]
$ns namtrace-all $nf

# definisco la procedura di chiusura
# 1. invoca la fine della simulazione
# 2. chiude le statistiche
# 3. lancia NAM
# 4. termina la simulazione
proc finish {} {
	global ns nf
	$ns flush-trace
	close $nf
	exec nam out.nam &
	exit 0
}

# descrivo la topologia della rete
# creo 4 nodi
set n0 [$ns node]
set n1 [$ns node]
set n2 [$ns node]
set n3 [$ns node]

# tipo link  // da // a // capacita' // ritardo // coda router  
# creo 3 link
$ns duplex-link $n0 $n2 1Mb 10ms DropTail
$ns duplex-link $n1 $n2 1Mb 10ms DropTail
# posso cambiare la politica di gestione della coda 
# cambiando la dichiarazione del link
$ns duplex-link $n3 $n2 1Mb 10ms SFQ
#$ns duplex-link $n3 $n2 1Mb 10ms DropTail

# visualizzo la coda in uscita da n2
$ns duplex-link-op $n2 $n3 queuePos 0.5

# nel modello ISO/OSI e in base alla terminologia TCP/IP
# nodi : livello 3 (trasmissione/ricezione di pacchetti e routing)
# link : livello 2
# host : livello 4 (trasporto) + livello applicazione 
#         (creazione/ricezione di dati)

# sorgente 1 UDP/CBR
# creo un agente UDP su n0 associandovi una classe
set udp0 [new Agent/UDP]
$udp0 set class_ 1 
$ns attach-agent $n0 $udp0
# creo un'applicazione sopra l'agente UDP di n0
# sorgente di traffico Constant BitRate (CBR) con pacchetti
# da 500 byte ogni 5ms
set cbr0 [new Application/Traffic/CBR]
$cbr0 set packetSize_ 500
$cbr0 set interval_ 0.005
$cbr0 attach-agent $udp0
#-- sorgente 1 UDP/CBR

# sorgente 2 UDP/CBR
# creo un agente UDP su n1 associandovi una classe
set udp1 [new Agent/UDP]
$udp1 set class_ 2
$ns attach-agent $n1 $udp1
# creo un'applicazione sopra l'agente UDP di n1
# sorgente di traffico Constant BitRate (CBR) con pacchetti
# da 500 byte ogni 5ms
set cbr1 [new Application/Traffic/CBR]
$cbr1 set packetSize_ 500
$cbr1 set interval_ 0.005
$cbr1 attach-agent $udp1
#-- sorgente 2 UDP/CBR

# creo un agente collettore di pacchetti su n2
# n2 fungera' da router
set null0 [new Agent/Null]
$ns attach-agent $n2 $null0

# creo un agente collettore di pacchetti su n3
set null1 [new Agent/Null]
$ns attach-agent $n3 $null1

# connetto gli agenti 
# n.b. livello 4 host-to-host, non e' necessario che 2 i rispettivi nodi siano 
# fisicamente adiacenti
$ns connect $udp0 $null1
$ns connect $udp1 $null1


# creo un'applicazione sopra l'agente UDP di n1
# sorgente di traffico Constant BitRate (CBR) con pacchetti
# da 500 byte ogni 5ms
set cbr1 [new Application/Traffic/CBR]
$cbr1 set packetSize_ 500
$cbr1 set interval_ 0.005
$cbr1 attach-agent $udp1

# con 'at' inserisco gli eventi di simulazione

# imposto la chiamata alla funzione di chiusura al 5' secondo di simulazione
$ns at 5.0 "finish"

# programmo l'accensione e lo spegnimento della sorgente CBR
$ns at 0.5 "$cbr0 start"
$ns at 1.0 "$cbr1 start"
$ns at 4.0 "$cbr1 stop"
$ns at 4.5 "$cbr0 stop"

# lancio la simulazione
$ns run 
