# instanzio un oggetto simulatore
set ns [new Simulator]

# definisco il tipo di simulazione
# !! e' solo per la versione di ns2 installata in laboratorio EDA
$ns set cosim_type 0

# creo i file per 'xgraph' con le statistiche
set f0 [open out0.tr w]
set f1 [open out1.tr w]
set f2 [open out2.tr w]

# definisco la procedura di chiusura
# 1. chiude le statistiche
# 2. lancia 'xgraph'
# 3. termina la simulazione
proc finish {} {
	global f0 f1 f2
	close $f0
	close $f1
	close $f2
	exec xgraph out0.tr out1.tr out2.tr -geometry 1680x1050 &
	exit 0
}

# descrivo la topologia della rete
# creo 5 nodi
set n0 [$ns node]
set n1 [$ns node]
set n2 [$ns node]
set n3 [$ns node]
set n4 [$ns node]

# tipo link  // da // a // capacita' // ritardo // coda router  
# creo 4 link
$ns duplex-link $n0 $n3 1Mb 100ms DropTail
$ns duplex-link $n1 $n3 1Mb 100ms DropTail
$ns duplex-link $n2 $n3 1Mb 100ms DropTail
$ns duplex-link $n3 $n4 1Mb 100ms DropTail

# nel modello ISO/OSI e in base alla terminologia TCP/IP
# nodi : livello 3 (trasmissione/ricezione di pacchetti e routing)
# link : livello 2
# host : livello 4 (trasporto) + livello applicazione 
#         (creazione/ricezione di dati)

# definisco una procedura per la creazione delle sorgenti
# generatore di traffico esponenziale su UDP
proc attach-expoo-traffic { node sink size burst idle rate } {
	set ns [Simulator instance]

	# creo un agente UDP sul nodo corrente	
	set source [new Agent/UDP]
	$ns attach-agent $node $source

	# creo un'applicazione, sorgente di traffico esponenziale
	set traffic [new Application/Traffic/Exponential]
	$traffic set packetSize_ $size
	$traffic set burst_time_ $burst
	$traffic set idle_time_ $idle
	$traffic set rate_ $rate

	# attacco l'applicazione all'agente
	$traffic attach-agent $source

        # connetto l'agente corrente all'agente destinazione
	$ns connect $source $sink
	return $traffic
}
#--

# creo 3 agenti destinatari 
set sink0 [new Agent/LossMonitor]
set sink1 [new Agent/LossMonitor]
set sink2 [new Agent/LossMonitor]
$ns attach-agent $n4 $sink0
$ns attach-agent $n4 $sink1
$ns attach-agent $n4 $sink2

# creo le sorgenti con traffico esponenziale
set source0 [attach-expoo-traffic $n0 $sink0 200 2s 1s 100k]
set source1 [attach-expoo-traffic $n1 $sink1 200 2s 1s 200k]
set source2 [attach-expoo-traffic $n2 $sink2 200 2s 1s 300k]

# definisco la procedura che salva su file i byte ricevuti dai tre destinatari
proc record {} {
	global sink0 sink1 sink2 f0 f1 f2
	set ns [Simulator instance]
        # fisso ogni quanto tempo la procedura viene richiamata
	set time 0.5
        # misuro quanti sono i byte ricevuti da ciascun destinatario
	set bw0 [$sink0 set bytes_]
	set bw1	[$sink1 set bytes_]
	set bw2 [$sink2 set bytes_]
	# recuperto il tempo corrente
	set now [$ns now]
	# calcolo la banda (MBit/s) e la scrivo su file
	puts $f0 "$now [expr $bw0/$time*8/1000000]"
	puts $f1 "$now [expr $bw1/$time*8/1000000]"
	puts $f2 "$now [expr $bw2/$time*8/1000000]"
	# resetto il contatore per ciascun destinatario
	$sink0 set bytes_ 0
	$sink1 set bytes_ 0
	$sink2 set bytes_ 0
	# rischedulo la simulazione
	$ns at [expr $now+$time] "record"
}
 

# con 'at' inserisco gli eventi di simulazione

# imposto la chiamata alla funzione di chiusura al 5' secondo di simulazione
$ns at 60.0 "finish"

# programmo l'accensione e lo spegnimento della sorgente CBR
$ns at 0.0 "record"
$ns at 10.0 "$source0 start"
$ns at 10.0 "$source1 start"
$ns at 10.0 "$source2 start"
$ns at 50.0 "$source0 stop"
$ns at 50.0 "$source1 stop"
$ns at 50.0 "$source2 stop"

# lancio la simulazione
$ns run 
